#! /usr/bin/python
import json
import SocketServer
import BaseHTTPServer

PORT = 0
ADDRESS = "0.0.0.0"

class Server(SocketServer.ThreadingMixIn, BaseHTTPServer.HTTPServer):
    """Handle requests in a separate thread."""
    def __init__(self, *args, **kwargs):
        BaseHTTPServer.HTTPServer.__init__(self, *args, **kwargs)
        self.database = {}

    def send_headers(self, client):
        client.send_header("Access-Control-Allow-Origin", "*")
        client.send_header("Content-Type", "application/json")

    def port(self):
        return self.socket.getsockname()[1]

    def got_request(self, data, client_socket):
        """
        Receivces jsons from handlers and deals with them
        """
        response = None
        # Make sure there is a key
        if "key" in data:
            key = data["key"]
            # If there is a value then update the key
            if "value" in data:
                self.database[key] = data["value"]
            # Make sure there is a key to send back, else send nothing
            if key in self.database:
                # Send the value of the key back to the client
                response = {
                    "key": key,
                    "value": self.database[key],
                }
        # Dump the json into the client socket
        if response is not None:
            json.dump(response, client_socket)

class Handler(BaseHTTPServer.BaseHTTPRequestHandler):
    """
    Handles post requests
    """
    def log_message(self, *args):
        return

    def do_POST(self):
        """
        Sends a 200 to the client and starts a new thread to
        execute the proper hook function
        """
        try:
            # Get the length of the post data
            content_len = int(self.headers.getheader("content-length", 0))
            # Read the post data
            post_body = self.rfile.read(content_len)
            # Load the post data from its json form to a dict
            post_body = json.loads(post_body)
            # Send the client a success reponse
            self.send_response(200)
            self.server.send_headers(self)
            self.end_headers()
            # Deal with the recived json
            self.server.got_request(post_body, self.wfile)
        except Exception as error:
            self.send_response(501)
            self.server.send_headers(self)
            self.end_headers()
            self.wfile.write(str(error) + "\n")
        return



import json
import urllib2

import server

HOST = "localhost"

class client(object):
    def __init__(self, host=HOST, port=server.PORT, protocol="http"):
        super(client, self).__init__()
        self.host = host
        self.port = port
        self.protocol = protocol

    def post(self, data):
        url = "{0}://{1}:{2}/".format(self.protocol, self.host, self.port)
        req = urllib2.Request(url, data, {"Content-Type": "application/json"})
        response = urllib2.urlopen(req)
        return response

    def send(self, data):
        key = data["key"]
        data = json.dumps(data)
        response = self.post(data)
        try:
            response = json.load(response)
        except Exception:
            response = {
                "key": key,
                "value": None,
            }
        return response

    def set(self, key, value):
        data = {
            "key": key,
            "value": value,
        }
        response = self.send(data)
        return response["value"]

    def get(self, key):
        data = {
            "key": key,
        }
        response = self.send(data)
        return response["value"]

    def __call__(self, key, value=False):
        if value:
            return self.set(key, value)
        return self.get(key)



from distutils.core import setup
setup(
  name = "qson",
  packages = ["qson"], # this must be the same as the name above
  version = '0.0.3',
  description = "Quick JSON based key value store over http",
  author = "John Andersen",
  author_email = "johnandersenpdx@gmail.com",
  url = "https://github.com/pdxjohnny/qson", # use the URL to the github repo
  download_url = "https://github.com/pdxjohnny/qson/tarball/0.0.1",
  keywords = ["server", "http", "https", "ssl"], # arbitrary keywords
  classifiers = [],
)



"""
qson

Quick JSON based key value store over http
"""
import os
import sys

import server
import client

VERSION = "0.0.3"

def make_daemon():
    # Daemonize to run in background
    pid = os.fork()
    if pid > 0:
        # exit first parent
        sys.exit(0)
    pid = os.fork()
    if pid > 0:
        # exit second parent
        sys.exit(0)
    else:
        output = open("/dev/null", 'wb')
    sys.stdout = output
    sys.stderr = output

def start(port=server.PORT, daemon=False):
    """
    Starts the webserver, first argument is port number, default is 9898
    """
    if type(port) != int:
        port = int(port)
    address = (server.ADDRESS, port)
    test_server = server.Server(address, server.Handler)
    print test_server.port()
    if daemon is not False:
        make_daemon()
    test_server.serve_forever()

def method(*args):
    """
    Runs client methods
    """
    test_client = client.client(port=args[0])
    action = getattr(test_client, args[1])
    print action(*args[2:])

def query(*args):
    """
    Runs client get or set
    """
    test_client = client.client(port=args[0])
    print test_client(*args[1:])

def main():
    action = getattr(sys.modules[__name__], sys.argv[1])
    action(*sys.argv[2:])

if __name__ == '__main__':
    main()



import sys
import thread

import server
import client

test_server = server.Server(("0.0.0.0", 0), server.Handler)
port = test_server.port()

thread.start_new_thread(test_server.serve_forever, tuple())

test_client = client.client(port=port)
print test_client("key1")
print test_client("key2", {
    "test": "values"
})
print test_client("key2")
